function [loglik, s_up, error, sol, sysm] = evalmod_kalman_gensys_sv(param, sigt, YY, estinfo, modelname)

% housekeeping
nobs = size(YY,1);
error = 0;
sol = [];

% sysmat for kalman
order_app = 1;

% evaluate sysmat
eval(['[DD,ZZ,HH,TT,RR,QQ,gensol] = ', ...
    modelname, 'sysmat(param, order_app, estinfo, modelname);']);
verysmall = estinfo.zerocleaning;
% CC1(abs(CC1)<verysmall) =0;
% CC2(abs(CC2)<verysmall) =0;
TT(abs(TT)<verysmall)   =0;
RR(abs(RR)<verysmall)   =0;

% /**********************************************************
% ** Check determinacy 
% eu(1)=1 for existence, eu(2)=1 for uniqueness.  eu(1)=-1 for
% existence only with not-s.c. z; eu=[-2,-2] for coincident zeros.
% **********************************************************/
if (gensol.RC(1) == 1) && (gensol.RC(2)==1);
   %/* determinacy */
   error = 0;
elseif (gensol.RC(1) == 1) && (gensol.RC(2)==0);
   %/* indeterminacy */
   error = 1;
else
   %/* no equilibrium exists, numerical problems */
   error = 2;
end

% Sungbae An's KF
sysm.DD = DD;
sysm.ZZ = ZZ;
sysm.HH = sqrt(HH); %standard dev

sysm.CC = zeros(size(TT,1),1);
sysm.TT = TT;
sysm.RR = RR*sqrt(QQ); 
sysm.QQ = eye(size(QQ,1)); %no QQ

if error ==0
% initial mean and variance for state vector
% [s00, P00] = initialize_kf_sv(size(sysm.TT,1),sysm.TT,sysm.RR,sigt,modelname);
[s00, P00, sysm.tg0] = initialize_kf(size(sysm.TT,1),sysm.TT,sysm.RR,sysm.QQ,modelname,gensol); % assume SV is in ss (ht=0)
% call Kalman filter
if estinfo.infle == 0
    [loglikiii,filtmv,loglik]=kalmanf_sv(sysm,s00,P00,YY,sigt);
else
    sysm.tg0 = gensol.tg0; %for later
%     if estinfo.zST == 1
%         sysm.zz0 = gensol.zz0; %for later
%     end
    [loglikiii,filtmv,loglik]=kalmanf_infle_sv(sysm,s00,P00,YY,sigt);
end
s_up = filtmv.s0;

% to generate prediction
if estinfo.predXlast == 1
    sysm.Xlastfix  = filtmv.s0(:,end);
    sysm.Xlastdraw = filtmv.s0(:,end) + sqrtSVD(filtmv.PT)*randn(size(sysm.TT,1),1);
    sysm.XlastPT   = filtmv.PT;
end
end

% save solution end
% ---------------------
% error from sysmat
if error ~= 0
    loglik =  -inf;
    s_up = [];
    sysm = [];
    return;    
end

% error control
if any(~isreal(loglik))
    error = 20;
    loglik = -inf;
    s_up = [];
    sysm = [];
end


% 
% % % ------------------------------------------------
% % % Kalman filter parametrication for the function
% % [ny, nx] = size(PP1G);
% % ZZ1 = ZZ(:,1:ny);
% % ZZ2 = ZZ(:,ny+1:end);
% % A   = DD + ZZ1*PP0G;
% % B   = ZZ1*PP1G+ZZ2;
% % H   = HH; % HH is sqrt matrix!
% % Phi = PP1H;
% % R   = sigeta;
% % S2  = eye(size(sigeta,2));
% % % initialization
% % [x0, P0] = initialize_kf(nx,Phi,R,S2,modelname);
% % [s_up, P_up, loglik] = kalman_filter_ini(x0,P0,A,B,Phi,R,H*H,S2,YY);
% % % ------------------------------------------------
% 
% % --------------------------------
% % An and Schorf way
% % write down the state vector in the following ways:
% % state = [y_{t}, x_{a,t+1}, x_{b,t}]
% % where x_{b,t} is exogenous shock
% A = DD;
% B = ZZ;
% H = HH;
% 
% [ny, nx] = size(PP1G);
% nxb = size(sigeta,2); %# of exogenous
% nxa = nx-nxb;
% 
% Ga = PP1G(:,1:nxa);
% Gb = PP1G(:,nxa+1:end);
% Haa = PP1H(1:nxa,1:nxa);
% Hab = PP1H(1:nxa, nxa+1:end);
% Hbb = PP1H(nxa+1:end, nxa+1:end);
% 
% % here state = [y_{t}, x_{a,t+1}, x_{b,t}], strange timing...
% Phi = ...
%     [zeros(ny,ny),  Ga, Gb*Hbb; ...
%      zeros(nxa,ny), Haa, Hab*Hbb; ...
%      zeros(nxb,ny), zeros(nxb,nxa), Hbb];
%  
% R0  = sigeta(nxa+1:end,:);
% R   = [Gb*R0; Hab*R0; R0];
% S2  = eye(nxb);
% 
% % delete consumption from the state
% B   = B(:,2:end);
% Phi = Phi(2:end,2:end);
% R   = R(2:end,:);
% 
% if strcmp(estinfo.kf, 'mc')
% %     % my KF
% %     sysm = [];
% %     [x0, P0] = initialize_kf(size(Phi,1),Phi,R,S2,modelname);
% %     [s_up, P_up, loglik] = kalman_filter_ini(x0,P0,A,B,Phi,R,H*H,S2,YY);
% elseif strcmp(estinfo.kf,'an')
%     % An's filter
%     sysm.DD = A;
%     sysm.ZZ = B;
%     sysm.HH = H;
%     sysm.CC = zeros(size(Phi,1),1);
%     sysm.TT = Phi;
%     sysm.RR = R;
%     % initial mean and variance for state vector
%     [s00, P00] = initialize_kf_sv(size(sysm.TT,1),sysm.TT,sysm.RR,sigt,modelname);
%     % call Kalman filter
%     [loglikiii,filtmv,loglik]=kalmanf_sv(sysm,s00,P00,YY,sigt);
%     s_up = filtmv.s0;
% elseif strcmp(estinfo.kf,'an2')
% %     % An's filter
% %     sysm.DD = A;
% %     sysm.ZZ = B;
% %     sysm.HH = H;
% %     sysm.CC = zeros(size(Phi,1),1);
% %     sysm.TT = Phi;
% %     sysm.RR = R;
% %     % initial mean and variance for state vector
% %     [s00, P00] = initialize_kf(size(sysm.TT,1),sysm.TT,sysm.RR,eye(nxb),modelname);
% %     %[s00,P00] = initmv(sysm.TT,sysm.RR);
% %     % call Kalman filter
% %     [loglikiii,filtmv]=kalmanf_mc1(sysm,s00,P00,YY);
% %     loglik = loglikiii/nobs*ones(nobs,1);
% %     s_up = filtmv.s0; 
% end
% 
% 
% 
% if any(~isreal(loglik))
%     error = 20;
%     loglik = -inf;
%     s_up = [];
%     sysm = [];
% end
% 
% 
% 
% 
% 
% 
% 























