function gensol = HB2012ST_solve(param, order_app, estinfo, modelname)
% solve DSGE model - Using GENSYS
% NK2: complicated NK
% HA : habit
% GR : growth rule
% IT means: Inflation target is time-varying

% /***************************************************************
% ** Solve DSGE using GENSYS
% ***************************************************************/
% return codes
% retcode = 0;
% valid   = 1;

% /***************************************************************
% ** Assign names to parameters
% ***************************************************************/
precover; %script to recover parameters, 'param' and 'estinfo.specname' is needed

%% Steady state computation
pist   = (1 + pist/400);   %qoq inflation
% pistst = (1 + pistst/400); %qoq price indexation
% bet    = 1/(1+rrst/400);
% gamst  = gamst/100; %gam is already defined in log
bet    = 1/(1+rrst/400);

% /***************************************************************
% ** Matrices of canonical system
% ***************************************************************/
% /** equation indices **/
eq_f1  = 1;
eq_f2  = 2;
eq_f3  = 3;
eq_f4  = 4;
eq_f5  = 5;

eq_f6  = 6;
eq_f7  = 7;
eq_f8  = 8;
eq_f9  = 9;
eq_f10 = 10;
eq_f11 = 11;
% eq_f12 = 12;
% eq_f13 = 13;

% /** variable indices **/
C_t       = 1;
PI_t      = 2;
R_t       = 3;
MC_t      = 4;
G_t       = 5;
Y_t       = 6;
Ylag_t    = 7;
% Ztil_t    = 8;
% Ztillag_t = 9;
Z_t       = 8;
EC_t      = 9;
EZ_t      = 10;
EPI_t     = 11;


% /** shock indices (neps) **/
MP_sh = 1;
G_sh  = 2;
Z_sh = 3;

% e_R = 1;
% e_g = 2;
% e_z = 3;
% e_trgt = 4; %shock to inflation target

% /** expectation error indices (neta) **/
EC_sh   = 1 ;
EPI_sh  = 2 ;
EZ_sh   = 3 ;

% n_pi = 1;
% n_y  = 2;
% n_g  = 3;
% n_trgt  = 4;

% /** summary **/
neq  = 11; %# of equation
neps = 3; %# of shocks
neta = 3; %# of expecations

% /** initialize matrices **/
GAM0 = zeros(neq,neq);
GAM1 = zeros(neq,neq);
   C = zeros(neq,1);
 PSI = zeros(neq,neps);
 PPI = zeros(neq,neta);

% f1: Euler
GAM0(eq_f1, C_t)  = 1;
GAM0(eq_f1, EC_t) = -1;
GAM0(eq_f1, EZ_t) = -1;
GAM0(eq_f1, R_t)  = 1/sig_C;
GAM0(eq_f1, EPI_t)  = -1/sig_C;
 
% f2: MC
GAM0(eq_f2, MC_t) = 1;
GAM0(eq_f2, C_t) = -1;
GAM0(eq_f2, Y_t) = -nu_l;

% f3: NKPC
GAM0(eq_f3, PI_t)  = 1;
GAM0(eq_f3, MC_t)  = -(1-zeta_p*bet)*(1-zeta_p)/(1+iota_p*bet)/zeta_p;
GAM0(eq_f3, EPI_t) = -bet/(1+iota_p*bet);

GAM1(eq_f3, PI_t)  = iota_p/(1+iota_p*bet); %lag

% f4: Taylor
% **  R_t = rho_R*R_t-1 + (1-rho_R)*[ psi_pi*PI_t + psi_y*(Y_t - Y_tlag + da_t)] + MP
% ***************************************************************/
GAM0(eq_f4, R_t)        = 1;
GAM0(eq_f4, PI_t)       = -( (1-rhor)*psi1 );
GAM0(eq_f4, Y_t)        = -( (1-rhor)*psi2 );
GAM0(eq_f4, Ylag_t)     = -( -(1-rhor)*psi2 );
GAM0(eq_f4, Z_t)       = -( (1-rhor)*psi2 );
GAM1(eq_f4, R_t)        = rhor;

PSI(eq_f4, MP_sh)       = 1;

% f5: resource
% /***************************************************************
% ** 10. Aggregate resource constraint (eq_res)
% **  Y_t = C_t + GOV_t
% ***************************************************************/
GAM0(eq_f5, Y_t)  = 1;
GAM0(eq_f5, C_t)  = -1;
GAM0(eq_f5, G_t)  = -1;

% f6: lag y
% **  Y_tlag = Y(-1)
% ***************************************************************/
GAM0(eq_f6, Ylag_t)  = 1;
GAM1(eq_f6, Y_t)    = 1;


% /***************************************************************
% ** 12. Target inflation rate (eq_PItarget)
% **  PItarget = PItarget(-1) + PItarget_sh
% ***************************************************************/
% // We shut down inflation target dynamics here
% //GAM0[eq_PItarget, PItarget_t] = 1;
% //GAM1[eq_PItarget, PItarget_t] = 1;
% //PSI[eq_PItarget, PItarget_sh] = 1;

% /**********************************************************
% **      Shock process
% **********************************************************/
% /** g **/
GAM0(eq_f7,G_t)  = 1;
GAM1(eq_f7,G_t)  = rhog;
 PSI(eq_f7,G_sh) = 1;

% /** z **/
GAM0(eq_f8,Z_t) = 1;
GAM1(eq_f8,Z_t) = -(varphi_z);
PSI(eq_f8,Z_sh) = 1;

% if rhoz ~=1
%     GAM0(eq_f8,Ztil_t) = 1;
%     GAM1(eq_f8,Ztil_t) = rhoz*(1-varphi_z);
%     GAM1(eq_f8,Ztillag_t) = varphi_z;
%     PSI(eq_f8,Z_sh) = 1;
% elseif rhoz ==1
%     GAM0(eq_f8,Z_t) = 1;
%     GAM1(eq_f8,Z_t) = -(varphi_z);
%     PSI(eq_f8,Z_sh) = 1;
% end

% % Ztil lag
% GAM0(eq_f9,Ztillag_t) = 1;
% GAM1(eq_f9,Ztil_t) = 1;
% 
% % Zt
% GAM0(eq_f10,Z_t) = 1;
% GAM0(eq_f10,Ztil_t) = -1;
% GAM1(eq_f10,Ztil_t) = -1;


% % /** MP **/
% THIS DEFINED IN THE R_t EQUATION
% GAM0(eq_MP,MP_t)  = 1;
% PSI(eq_MP,R_sh) = 1;
 
% /**********************************************************
% **      Expectation error
% **********************************************************/
% /** E[C] **/
GAM0(eq_f9, C_t)  = 1;
GAM1(eq_f9, EC_t) = 1;
PPI(eq_f9, EC_sh) = 1;

% /** E[Z] **/
GAM0(eq_f10, Z_t)  = 1;
GAM1(eq_f10, EZ_t) = 1;
PPI(eq_f10, EZ_sh) = 1;

% /** E[PI] **/
GAM0(eq_f11, PI_t)  = 1;
GAM1(eq_f11, EPI_t) = 1;
PPI(eq_f11, EPI_sh) = 1;


 
% /***************************************************************
% ** QZ(generalized Schur) decomposition by GENSYS
% ***************************************************************/
% [T1, TC, T0, TY, M, TZ, TETA, GEV, RC] = gensys(GAM0, GAM1, C, PSI, PPI, 1+1E-8,1);
[T1,TC,T0,M,TZ,TY,GEV,RC,TETA]= gensys_minchul(GAM0,GAM1,C,PSI,PPI,1+1E-8,estinfo.ordqz);

gensol.T1 = T1;
gensol.TC = TC;
gensol.T0 = T0;
gensol.M  = M;
gensol.TZ = TZ;
gensol.TY = TY;
gensol.GEV = GEV;
gensol.RC  = RC;
gensol.TETA = TETA;

% if valid == 0
%     gensol.RC(1) = 100;
% end

