function [DD,ZZ,HH,TT,RR,QQ,gensol,tg0] = HB2012ST_trgtar_sbsv_sysmat(param, order_app, estinfo, modelname)
% FOR GENSYS
% IT means: TV-inflation target, tg0 is initial target rate.
%% SOLVE using GENSYS
eval(['gensol = ', modelname, 'solve(param, order_app, estinfo, modelname);']);

%% SYSTEM MATRICES - setup
% /** observation indices **/
eq_ygr = 1;
eq_pi  = 2;
eq_r   = 3;
eq_TG = 4;

% /** number of observation variables **/
ny = 4;

% /** variable indices **/
C_t       = 1;
PI_t      = 2;
R_t       = 3;
MC_t      = 4;
G_t       = 5;
Y_t       = 6;
Ylag_t    = 7;
% Ztil_t    = 8;
% Ztillag_t = 9;
Z_t       = 8;
EC_t      = 9;
EZ_t      = 10;
EPI_t     = 11;
TG_t      = 12; %inflation target

% /** shock indices (neps) **/
e_R = 1;
e_g = 2;
e_z = 3;
e_TG = 4;
% e_trgt = 4; %shock to inflation target

% RECOVER PARAMETERS
precover; %script to recover parameters, 'param' and 'estinfo.specname' is needed

%% SYSTEM MATRICES - Transition
% /*************************************************
% **       Transition equations with augmentation
% **
% **       z(t) = T1*z(t-1) + T0*e(t)
% **       x(t) = [z(t)',...]'
% **
% **       x(t) = TT*x(t-1) + RR*e(t)
% **       e(t) ~ iid N(0,QQ)
% *************************************************/
nz  = size(gensol.T1,1);
nep = size(gensol.T0,2);

% We augment v_ylag (defined in the program)
% v_ylag = nz+1;

TT = gensol.T1;
% TT = zeros(nz+1,nz+1);
% TT(1:nz,1:nz)   = gensol.T1;
% TT(v_ylag, v_y) = 1;

QQ = zeros(nep,nep);
QQ(e_R, e_R) = 1;
QQ(e_g, e_g) = 1;
QQ(e_z, e_z) = 1;
QQ(e_TG,e_TG) = 1;
% QQ(e_R, e_R) = sigr^2;
% QQ(e_g, e_g) = sigg^2;
% QQ(e_z, e_z) = sigz^2;
% QQ(e_TG,e_TG) = sigtg^2;

RR = gensol.T0;
% RR = [gensol.T0; zeros(1,nep)];

%% SYSTEM MATRICES - Measurement
% /*************************************************
% ** Measurement equations
% **
% **  Ygr_US = gam_steady + Y - Y(-1) + Z;
% **  INF_US = pi_steady + 4*PI;
% **  R_US   = pi_steady + rr_steady + 4*gam_steady+ 4*R;
% **
% **  y(t) = DD + ZZ*x(t) + u(t)
% **  u(t) ~ N(0,HH)
% **  
% **  cov(e(t),u(t)) = VV
% *************************************************/
ns = size(TT,1);

DD = zeros(ny,1);

DD(eq_ygr,1) = gamst;
DD(eq_pi,1)  = pist;
DD(eq_r,1)   = pist+rrst+4*gamst;
DD(eq_TG,1)  = pist;

ZZ = zeros(ny,ns);

ZZ(eq_ygr, Y_t)    = 1;
ZZ(eq_ygr, Ylag_t) = -1;
ZZ(eq_ygr, Z_t)    = 1;
ZZ(eq_pi,  PI_t)   = 4;
ZZ(eq_r,   R_t)    = 4;

ZZ(eq_TG, TG_t)    = 4;

% new measurement equation for trgt model (following DS)
if estinfo.trgt_measurement == 1

    ZZ(eq_TG,:) = nan;
    %II = eye(size(TT));
    %INFLexp10 = ( (II-TT)\(II-TT^40)*TT );
    
    INFLexp10 = 0;
    for iii = 1:1:40
        INFLexp10 = INFLexp10 + TT^iii;
    end
    
    
    ZZ(eq_TG,:) = 4/40*ZZ(eq_pi,:)*INFLexp10; %4 for annulatized data, 1/40 for 10 year average
    
    
    
end

HH = zeros(ny,ny);

HH(eq_ygr, eq_ygr) = sigmy^2;
HH(eq_pi, eq_pi)   = sigmp^2;
HH(eq_r, eq_r)     = sigmr^2;
HH(eq_TG, eq_TG)   = sigmtg^2;

% initial target rate
gensol.tg0 = tg0;
% gensol.zz0 = zz0;

gensol.QQ0 = zeros(nep,nep);
gensol.QQ0(e_R, e_R) = sigr0^2;
gensol.QQ0(e_g, e_g) = sigg0^2;
gensol.QQ0(e_z, e_z) = sigz0^2;
% gensol.QQ0(e_TG, e_TG) = sigtg0^2;

gensol.QQ1 = zeros(nep,nep);
gensol.QQ1(e_R, e_R) = sigr^2;
gensol.QQ1(e_g, e_g) = sigg^2;
gensol.QQ1(e_z, e_z) = sigz^2;
gensol.QQ1(e_TG, e_TG) = sigtg^2;





