% dsge_pm.m

% - sbsv means structural break in variance

% - script to max posterior density
n_particle = 0;
% ------------------------------------------------------------------------
% House cleaning and set defaults
% ------------------------------------------------------------------------
% set clock
time0 = now;

% start log file
whattimeisit = clock;
logfilename = ['log_pm', num2str(floor(whattimeisit(1,2:6))), '.log'];
cd(opath);
diary(logfilename);
cd(workpath);

% ------------------------------------------------------------------------
% Import prior specification
% ------------------------------------------------------------------------
% priorfile = [specname, '.txt'];
% cd(priopath);
% prior     = load(priorfile, '-ascii');
% cd(workpath);
% [prior, para_names] = psel(prior, msel);

% load prior
eval(['[prior, paraname] = prior_', specname,';'])

pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1- pmask;
pshape   = pshape.*pmaskinv;

% starting value for the optimization
try
    load(prepmpath);
    para = pmod;
    disp('Previous pmod is used as a starting value... ');
catch
    para  = pmsv(modelname, paraname);
end
% load('an_result_all.mat');
% para = pmod;

npara = size(para,1);
para  = para.*pmaskinv + pfix.*pmask;

% parameter specification: transformation
trspec      = prtr(modelname, paraname);
trspec(:,1) = trspec(:,1).*pmaskinv;

% ------------------------------------------------------------------------
% Calculate posterior density at starting values
% ------------------------------------------------------------------------
disp('-------------------------------------');
disp(['Model          ', modelname]);
disp(['Prior          ', lprior]);
disp(['Dataset        ', ldata]);
disp(['Order          ', num2str(order_app)]);
disp(['# of particles ', num2str(n_particle)]);
disp('-----------------------------');

disp(['Prior Density at Starting Value:']);
disp(num2str(priodens(para, pmean, pstdd, pshape)));
tic;
lnpy = -objfcn_pm_sbsv(invtrans(para, trspec), trspec, prior, estinfo, YY, modelname, order_app, n_particle, T0_);
disp('Posterior at Starting Value:    ');
disp(num2str(lnpy));
disp(['Posterior Evaluation Takes : ', num2str(toc), 'second']);

% ------------------------------------------------------------------------
% Maximize the posterior density: Sims' Routine
% - <NOTE>
%  Sometimes we need to change the scale in the numgrad function to get
%  reasonable results. Check the sensitivity of the objetive function
% ------------------------------------------------------------------------
x0 = invtrans(para, trspec);
H0 = eye(npara)*1e-4;

% Set optimization routine enviroment
nit  = 1000; % maximum number of iteration
crit = 1e-10; % tolerance

global delta_
delta_ = 1e-7;
% Use Sims' optimizer
[fh,xh,gh,H,itct,fcount,retcodeh] = ...
    csminwel('objfcn_pm_sbsv', x0, H0, [], crit, nit, ...
    trspec, prior, estinfo, YY, modelname, order_app, n_particle, T0_);

sims_fh   = fh;
sims_H    = H;
sims_xh   = xh;
pmod_sims = trans(xh, trspec);

% gradient at pmod
gg = zeros(npara,1);
for i=1:1:npara
[   gg(i,1), badg] = numgrad('invtrans',pmod_sims(i),trspec(i,:));
end
gradmod = gh./gg;


% ------------------------------------------------------------------------
% Maximize the posterior density: CMAES
% - <NOTE>
%  Sometimes we need to change the scale in the numgrad function to get
%  reasonable results. Check the sensitivity of the objetive function
% ------------------------------------------------------------------------
% I will fix it later...


% ------------------------------------------------------------------------
% Save parameter estimates and return code
% ------------------------------------------------------------------------
% Posterior Mode: max to model parameters
% pmod  = trans(xh, trspec);
% pmod  = pmod.*pmaskinv + pfix.*pmask;
% paraests = pmod';

pmod = pmod_sims;
% Posterior and Prior Value at paraeast
pmod_post  = -objfcn_pm_sbsv(invtrans(pmod_sims, trspec), trspec, prior, estinfo, YY, modelname, order_app, n_particle, T0_);
pmod_prior = priodens(pmod_sims, pmean, pstdd, pshape);


% ------------------------------------------------------------------------
% Estimation output
% ------------------------------------------------------------------------
% print out
disp('Model Spec  ------------------------------');
disp(['Model          ', modelname]);
disp(['Prior          ', lprior]);
disp(['Dataset        ', ldata]);
disp(['Order          ', num2str(order_app)]);
disp(['# of particles ', num2str(n_particle)]);
% disp(['Subsample     ', subTstr]);
disp('Log of -----------------------------------');
disp(['Posterior Mode       : ', num2str(pmod_post)]);
disp(['Likelihood at Mode   : ', num2str(pmod_post-pmod_prior)]);
disp(['Prior at Mode        : ', num2str(pmod_prior)]);
disp('------------------------------------------');
disp('------------------------------------------');
disp(['Return code(cminwel) : ', num2str(retcodeh)]);
disp('------------------------------------------');
disp('Esimation Results');
disp('------------------------------------------');
disp([{'parameters', 'mode', 'start', 'gradient'}; ...
[paraname num2cell(fixdec(pmod,3)) num2cell(fixdec(trans(x0, trspec),3)) num2cell(fixdec(gradmod,5))]]);
disp('------------------------------------------');

% calculate elapsed time and print into log
time1 = now;

disp(' ');
disp(' ');
disp('-------------------------------------------------------');
disp(['start       :  ' datestr(time0)]);
disp(['end         :  ' datestr(time1)]);
disp(['elapsed time:  ' datestr(time1-time0,13)]);
disp('-------------------------------------------------------');
disp(' ');

% off log file
diary off

% save some matrces
cd(opath);
% save('pm.mat', 'pmod', 'pmod_post', 'pmod_prior', 'penalt', 'sigmult', 'retcodeh');
save('pm.mat', 'pmod', 'pmod_post', 'pmod_prior', 'retcodeh', 'gradmod');
cd(workpath);



% pmod2 = pmod;
% pmax2 = pmod_post;
% plik2 = pmod_post-pmod_prior;
% 
% 
% pprio3 = priodens(pmod, pmean, pstdd, pshape);
% pmax3  = -objfcn_pm(invtrans(pmod, trspec), trspec, prior, estinfo, YY, modelname, order_app, n_particle, T0_);











